/*
 * Copyright (c) 2012-2014 Kaarle Ritvanen
 * See LICENSE file for license details
 */

define(
    [
	"acf2/widget/abstract/node",
	"jquery",
	"underscore",
	"acf2/widget/audio",
	"acf2/widget/checkbox",
	"acf2/widget/checkboxes",
	"acf2/widget/combobox",
	"acf2/widget/date",
	"acf2/widget/field",
	"acf2/widget/inline",
	"acf2/widget/reference"
    ],
    function(Base, $, _) {
	return Base.extend({
	    render: function(data, meta) {
		Base.render.call(this, data, meta);

		this.reqData = data;
		var self = this;

		if (meta.type == "model") {
		    _.each(meta.actions, function(action) {
			self.addActionButton(action["ui-name"], function() {
			    data.invoke(action.name)
				.done(function() { alert("Done"); })
				.fail(function() { alert("Failed"); });
			});
		    });

		    this.createModelWidgets(
			_.filter(meta.fields, function(field) {
			    return field.visible;
			}),
			true
		    );

		    var labels = _.object(_.map(meta.fields, function(field) {
			return [field.name, field["ui-name"]];
		    }));

		    _.each(this.widgets, function(f1, name) {
			self.setupWidget(f1, labels[name]);

			_.each(self.widgets, function(f2) {
			    if (f1 != f2)
				f1.on("validated", function(event) {
				    f2.trigger("updated", [name]);
				    event.stopPropagation();
				});
			});
		    });

		    _.each(this.widgets, function(widget) {
			widget.trigger("start");
		    });
		}

		else _.each(data.data, function(value, name) {
		    if (meta.type == "set") name = data.data[name];
		    else if (_.isArray(data.data)) name++;
		    self.renderCollectionMember(name, meta);
		});
	    },

	    createModelWidgets: function(fields, editable) {
		this.widgets = {};
		var self = this;
		_.each(fields, function(field) {
		    var widget = self.createWidget(
			field.name, field, editable, false
		    );
		    if (widget) self.widgets[field.name] = widget;
		});
	    },

	    createWidget: function(name, meta, editable, removable) {
		var widget = this.widget(meta);
		if (widget)
		    return widget.new(
			this.reqData,
			name,
			meta,
			this.level,
			editable,
			removable
		    );
	    },

	    setupWidget: function(widget, label) {
		var container = this.appendWidget(widget, label);
		widget.on("setVisible", function(event, visible) {
		    if (visible) container.show();
		    else container.hide();
		    event.stopPropagation();
		});
	    },

	    renderCollectionMember: function(name, meta) {
		var set = meta.type == "set";
		var widget = this.createWidget(
		    name,
		    meta.members,
		    !set,
		    !set && _.contains(meta.removable, name)
		);
		this.setupWidget(widget, meta["ui-member"] + " " + name);
		widget.trigger("start");
	    },

	    widget: function(meta) {
		return require("acf2/widget/" + meta.widget);
	    },

	    validate: function(data) {
		Base.validate.call(this, data);

		if (this.widgets)
		    _.each(this.widgets, function(widget) {
			widget.trigger("updated");
		    });
	    }
	});
    }
);
